﻿using SubscriberServicesClient.Emma.Subscription.Service.Client;
using SubscriberServicesClient.Helpers;
using System;
using System.IO;
using System.ServiceModel.Security;

namespace SubscriberServicesClient
{
    public class WsCaller
    {
        private readonly string _endpointAddress;
        private readonly string _endpointName;
        private readonly string _userId;
        private readonly string _password;
        private readonly string _certName;

        public WsCaller(string endpointAddress, string endpointName, string userId, string password, string certName)
        {
            _endpointAddress = endpointAddress;
            _endpointName = endpointName;
            _userId = userId;
            _password = password;
            _certName = certName;
        }

        private EmmaSubscriptionServiceClient CreateSvcClient()
        {
            if (string.IsNullOrEmpty(_endpointName))
            {
                throw new Form1.NotLoggedInException();
            }
            var client = new EmmaSubscriptionServiceClient(_endpointName, _endpointAddress);
            if(!string.IsNullOrEmpty(_certName))
            {
                client.ClientCredentials.ClientCertificate.Certificate = CertificateRepository.GetSelectedCertificate(_certName);
            }
            else
            {
                client.ClientCredentials.ServiceCertificate.Authentication.CertificateValidationMode = X509CertificateValidationMode.None;
                client.ClientCredentials.UserName.UserName = _userId;
                client.ClientCredentials.UserName.Password = _password;
            }
            return client;
        }

        private T Execute<T>(Func<EmmaSubscriptionServiceClient, T> f)
        {
            EmmaSubscriptionServiceClient svc = null;
            try
            {
                svc = CreateSvcClient();
                return f(svc);
            }
            finally
            {
                svc.Close();
            }
        }

        public EmmaTransactionList GetTransactions(string submissionType, DateTime postingDate, string specVersion)
        {
            return Execute(svc => svc.GetTransactions(submissionType, postingDate, specVersion));
        }

        public SingleEmmaTransaction GetTransaction(string submissionType, DateTime postingDate, string submissionId, string specVersion)
        {
            return Execute(svc => svc.GetTransaction(submissionType, postingDate, submissionId, specVersion));
        }

        public Stream GetFile(string fileId)
        {
            return Execute(svc => svc.GetFile(fileId));
        }
    }
}
